/*
 *  x48 port on PSP 
 *
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>
#include <stdarg.h>

# include <pspctrl.h>

# include "psp_init.h"
# include "psp_pg.h"
# include "psp_kbd.h"

#include "hp48.h"

/* Define the module info section */
PSP_MODULE_INFO("psp", 0, 1, 1);

/* Define the main thread's attribute value (optional) */
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

# ifdef DEBUG
  static FILE *pspDebugFd = (FILE *)0;

void 
pspDebugPrintf(const char *Format, ...)
{
  va_list  Args;
  va_start( Args, Format );
  if (pspDebugFd == (FILE *)0) {
    pspDebugFd = fopen("ms0:/psp/game/pspx48/debug.log", "w");
    if (! pspDebugFd) {
      pspDebugScreenPrintf("can\'t open debug file !\n");
      sceKernelExitGame();
    }
  }

  vfprintf(pspDebugFd, Format, Args);
  va_end( Args );
}
# endif


int
psp_exit(int status)
{
  save_emulator_stuff();
  if (status) {
    pspDebugScreenPrintf("exit with status %d!\n Press Start to contine\n", status);
    kbd_wait_start();
  }
# ifdef DEBUG
  if (pspDebugFd) {
    fclose(pspDebugFd);
    pspDebugFd = (FILE *)0;
  }
# endif
  sceKernelExitGame();
  return 0;
}

int
psp_exit_callback(int arg1, int arg2, void *arg3)
{
  pspDebugScreenPrintf("exit call back call !\n");
  psp_exit(0);

  return 0;
}

int
psp_power_callback(int arg1, int arg2, void *arg3)
{
  pspDebugScreenPrintf("power call back call !\n");
  psp_exit(0);

  return 0;
}

void
psp_init_stuff(void)
{
  int cid;

  cid = sceKernelCreateCallback("Exit Call Back", psp_exit_callback, NULL);
  sceKernelRegisterExitCallback(cid);
  cid = sceKernelCreateCallback("Power Call Back", psp_power_callback, NULL);
  scePowerRegisterCallback(0,cid);

  sceCtrlSetSamplingCycle(0);
  sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
}

